@@ -1,5 +1,5 @@
 #[cfg(target_os = "macos")]
-use crate::platform::macos::{IOKitRegistry, PowerMetrics};
+use crate::platform::macos::{CommandSource, IOKitRegistry};
 #[cfg(target_os = "windows")]
 use crate::platform::windows::{EtwTrace, Pdh, ProcessorInfo, ThermalZoneInformation, VmCounter};
 use crate::{Error, Pid};
@@ -9,10 +9,11 @@ use sysinfo::{ProcessExt, SystemExt};
 
 #[derive(Default)]
 pub struct System {
+    features: Features,
     sysinfo_system: Option<sysinfo::System>,
     refresh_kind: sysinfo::RefreshKind,
     #[cfg(target_os = "macos")]
-    power_metrics: Option<PowerMetrics>,
+    command_source: Option<CommandSource>,
     #[cfg(target_os = "macos")]
     ioreg: Option<IOKitRegistry>,
     #[cfg(target_os = "macos")]
@@ -34,6 +35,7 @@ impl System {
         pids: T,
     ) -> Result<Self, Error> {
         let mut system = System::default();
+        system.features = features;
 
         let mut use_sysinfo_system = false;
         if features.contains(Features::PROCESS) {
@@ -67,7 +69,10 @@ impl System {
         if features.contains(Features::CPU_FREQUENCY) {
             #[cfg(target_os = "macos")]
             {
-                system.power_metrics = Some(PowerMetrics::new());
+                system.command_source = Some(CommandSource::new(
+                    pids.clone(),
+                    features.contains(Features::NET_TRAFFIC),
+                ));
             }
             #[cfg(target_os = "windows")]
             {
@@ -78,7 +83,10 @@ impl System {
         if features.contains(Features::FPS) {
             #[cfg(target_os = "windows")]
             {
-                system.etw_trace = Some(EtwTrace::new());
+                system.etw_trace = Some(EtwTrace::new(
+                    true,
+                    features.contains(Features::NET_TRAFFIC),
+                ));
             }
         }
 
@@ -96,6 +104,25 @@ impl System {
             }
         }
 
+        if features.contains(Features::NET_TRAFFIC) {
+            #[cfg(target_os = "macos")]
+            {
+                system.command_source = Some(
+                    system
+                        .command_source
+                        .unwrap_or_else(|| CommandSource::new(pids, true)),
+                );
+            }
+            #[cfg(target_os = "windows")]
+            {
+                system.etw_trace = Some(
+                    system
+                        .etw_trace
+                        .unwrap_or_else(|| EtwTrace::new(false, true)),
+                );
+            }
+        }
+
         Ok(system)
     }
 
@@ -105,8 +132,13 @@ impl System {
         }
 
         #[cfg(target_os = "macos")]
-        if let Some(power_metrics) = &mut self.power_metrics {
-            power_metrics.poll();
+        if let Some(command_source) = &mut self.command_source {
+            if self.features.contains(Features::CPU_FREQUENCY) {
+                command_source.update_cpu_frequency();
+            }
+            if self.features.contains(Features::NET_TRAFFIC) {
+                command_source.update_net_traffic();
+            }
         }
 
         #[cfg(target_os = "macos")]
@@ -118,6 +150,11 @@ impl System {
         if let Some(pdh) = &mut self.pdh {
             pdh.update();
         }
+
+        #[cfg(target_os = "windows")]
+        if let Some(etw) = &mut self.etw_trace {
+            etw.update();
+        }
     }
 
     pub fn sysinfo_system(&self) -> Option<&sysinfo::System> {
@@ -164,7 +201,7 @@ impl System {
         #[cfg(target_os = "macos")]
         {
             Ok(self
-                .power_metrics
+                .command_source
                 .as_ref()
                 .ok_or(Error::FeatureMissing(Features::CPU_FREQUENCY))?
                 .cpu_frequency())
@@ -206,6 +243,28 @@ impl System {
         }
     }
 
+    pub fn process_net_traffic_in(&self, pid: Pid) -> Option<u32> {
+        #[cfg(target_os = "macos")]
+        {
+            self.command_source.as_ref()?.process_net_traffic_in(pid)
+        }
+        #[cfg(target_os = "windows")]
+        {
+            Some(self.etw_trace.as_ref()?.net_recv_per_sec(pid))
+        }
+    }
+
+    pub fn process_net_traffic_out(&self, pid: Pid) -> Option<u32> {
+        #[cfg(target_os = "macos")]
+        {
+            self.command_source.as_ref()?.process_net_traffic_out(pid)
+        }
+        #[cfg(target_os = "windows")]
+        {
+            Some(self.etw_trace.as_ref()?.net_send_per_sec(pid))
+        }
+    }
+
     pub fn system_gpu_usage(&mut self) -> Option<f32> {
         #[cfg(target_os = "macos")]
         {
@@ -262,12 +321,14 @@ impl System {
 }
 
 bitflags! {
+    #[derive(Default)]
     pub struct Features: u32 {
         const PROCESS =         1 << 0;
         const GPU =             1 << 1;
         const CPU_FREQUENCY =   1 << 2;
         const FPS =             1 << 3;
         const SMC =             1 << 4;
+        const NET_TRAFFIC =     1 << 5;
     }
 }
 
